@@ -140,11 +140,10 @@ static void *overlay_extension_mappings(apr_pool_t *p,
                                         const void *base_val,
                                         const void *data)
 {
-    extension_info *new_info = apr_palloc(p, sizeof(extension_info));
     const extension_info *overlay_info = (const extension_info *)overlay_val;
     const extension_info *base_info = (const extension_info *)base_val;
+    extension_info *new_info = apr_pmemdup(p, base_info, sizeof(extension_info));
 
-    memcpy(new_info, base_info, sizeof(extension_info));
     if (overlay_info->forced_type) {
         new_info->forced_type = overlay_info->forced_type;
     }
@@ -274,6 +273,16 @@ static const char *add_extension_info(cmd_parms *cmd, void *m_,
     return NULL;
 }
 
+/*
+ * As RemoveType should also override the info from TypesConfig, we add an
+ * empty string as type instead of actually removing the type.
+ */
+static const char *remove_extension_type(cmd_parms *cmd, void *m_,
+                                         const char *ext)
+{
+    return add_extension_info(cmd, m_, "", ext);
+}
+
 /*
  * Note handler names are un-added with each per_dir_config merge.
  * This keeps the association from being inherited, but not
@@ -313,6 +322,12 @@ static const char *multiviews_match(cmd_parms *cmd, void *m_,
                                     const char *include)
 {
     mime_dir_config *m = (mime_dir_config *) m_;
+    const char *errmsg;
+
+    errmsg = ap_check_cmd_context(cmd, NOT_IN_LOCATION);
+    if (errmsg != NULL) {
+        return errmsg;
+    }
 
     if (strcasecmp(include, "Any") == 0) {
         if (m->multimatch && (m->multimatch & ~MULTIMATCH_ANY)) {
@@ -398,7 +413,7 @@ static const command_rec mime_cmds[] =
     AP_INIT_ITERATE("RemoveOutputFilter", remove_extension_info,
         (void *)APR_OFFSETOF(extension_info, output_filters), OR_FILEINFO,
         "one or more file extensions"),
-    AP_INIT_ITERATE("RemoveType", remove_extension_info,
+    AP_INIT_ITERATE("RemoveType", remove_extension_type,
         (void *)APR_OFFSETOF(extension_info, forced_type), OR_FILEINFO,
         "one or more file extensions"),
     AP_INIT_TAKE1("TypesConfig", set_types_config, NULL, RSRC_CONF,
@@ -810,7 +825,8 @@ static int find_ct(request_rec *r)
 
         if (exinfo != NULL) {
 
-            if (exinfo->forced_type) {
+            /* empty string is treated as special case for RemoveType */
+            if (exinfo->forced_type && *exinfo->forced_type) {
                 ap_set_content_type(r, exinfo->forced_type);
                 found = 1;
             }
