@@ -561,7 +561,7 @@ int initgroups(const char *name, gid_t basegid)
     return setgroups(index, groups);
 #endif /* def QNX */
 }
-#endif /* def NEED_INITGROUPS */
+#endif /* def HAVE_INITGROUPS */
 
 #ifdef AP_MPM_USES_POD
 
