@@ -25,6 +25,8 @@
 #include "apr_strings.h"
 #include "apr_hash.h"
 #include "apr_thread_mutex.h"
+#include "apr_lib.h"
+#include "apr_atomic.h"
 
 #include "apu_internal.h"
 #include "apr_dbd_internal.h"
@@ -32,13 +34,14 @@
 #include "apu_version.h"
 
 static apr_hash_t *drivers = NULL;
+static apr_uint32_t initialised = 0, in_init = 1;
 
 #define CLEANUP_CAST (apr_status_t (*)(void*))
 
 #if APR_HAS_THREADS
 /* deprecated, but required for existing providers.  Existing and new
  * providers should be refactored to use a provider-specific mutex so
- * that different providers do not block one another. 
+ * that different providers do not block one another.
  * In APR 1.3 this is no longer used for dso module loading, and
  * apu_dso_mutex_[un]lock is used instead.
  * In APR 2.0 this should become entirely local to libaprutil-2.so and
@@ -89,16 +92,22 @@ APU_DECLARE(apr_status_t) apr_dbd_init(apr_pool_t *pool)
     apr_status_t ret = APR_SUCCESS;
     apr_pool_t *parent;
 
-    if (drivers != NULL) {
+    if (apr_atomic_inc32(&initialised)) {
+        apr_atomic_set32(&initialised, 1); /* prevent wrap-around */
+
+        while (apr_atomic_read32(&in_init)) /* wait until we get fully inited */
+            ;
+
         return APR_SUCCESS;
     }
 
     /* Top level pool scope, need process-scope lifetime */
     for (parent = pool;  parent; parent = apr_pool_parent_get(pool))
          pool = parent;
-
+#if APU_DSO_BUILD
     /* deprecate in 2.0 - permit implicit initialization */
     apu_dso_init(pool);
+#endif
 
     drivers = apr_hash_make(pool);
 
@@ -108,6 +117,7 @@ APU_DECLARE(apr_status_t) apr_dbd_init(apr_pool_t *pool)
 #endif
 
 #if !APU_DSO_BUILD
+
     /* Load statically-linked drivers: */
 #if APU_HAVE_MYSQL
     DRIVER_LOAD("mysql", apr_dbd_mysql_driver, pool);
@@ -124,6 +134,12 @@ APU_DECLARE(apr_status_t) apr_dbd_init(apr_pool_t *pool)
 #if APU_HAVE_ORACLE
     DRIVER_LOAD("oracle", apr_dbd_oracle_driver, pool);
 #endif
+#if APU_HAVE_FREETDS
+    DRIVER_LOAD("freetds", apr_dbd_freetds_driver, pool);
+#endif
+#if APU_HAVE_ODBC
+    DRIVER_LOAD("odbc", apr_dbd_odbc_driver, pool);
+#endif
 #if APU_HAVE_SOME_OTHER_BACKEND
     DRIVER_LOAD("firebird", apr_dbd_other_driver, pool);
 #endif
@@ -132,6 +148,8 @@ APU_DECLARE(apr_status_t) apr_dbd_init(apr_pool_t *pool)
     apr_pool_cleanup_register(pool, NULL, apr_dbd_term,
                               apr_pool_cleanup_null);
 
+    apr_atomic_dec32(&in_init);
+
     return ret;
 }
 
@@ -145,14 +163,17 @@ APU_DECLARE(apr_status_t) apr_dbd_get_driver(apr_pool_t *pool, const char *name,
 #endif
     apr_status_t rv;
 
+#if APU_DSO_BUILD
     rv = apu_dso_mutex_lock();
     if (rv) {
         return rv;
     }
-
+#endif
     *driver = apr_hash_get(drivers, name, APR_HASH_KEY_STRING);
     if (*driver) {
+#if APU_DSO_BUILD
         apu_dso_mutex_unlock();
+#endif
         return APR_SUCCESS;
     }
 
@@ -164,29 +185,23 @@ APU_DECLARE(apr_status_t) apr_dbd_get_driver(apr_pool_t *pool, const char *name,
 #if defined(NETWARE)
     apr_snprintf(modname, sizeof(modname), "dbd%s.nlm", name);
 #elif defined(WIN32)
-    apr_snprintf(modname, sizeof(modname), 
+    apr_snprintf(modname, sizeof(modname),
                  "apr_dbd_%s-" APU_STRINGIFY(APU_MAJOR_VERSION) ".dll", name);
 #else
-    apr_snprintf(modname, sizeof(modname), 
+    apr_snprintf(modname, sizeof(modname),
                  "apr_dbd_%s-" APU_STRINGIFY(APU_MAJOR_VERSION) ".so", name);
 #endif
     apr_snprintf(symname, sizeof(symname), "apr_dbd_%s_driver", name);
-    rv = apu_dso_load(&symbol, modname, symname, pool);
-    if (rv != APR_SUCCESS) { /* APR_EDSOOPEN or APR_ESYMNOTFOUND? */
-        if (rv == APR_EINIT) { /* previously loaded?!? */
-            apr_hash_set(drivers, name, APR_HASH_KEY_STRING, *driver);
-            rv = APR_SUCCESS;
+    rv = apu_dso_load(NULL, &symbol, modname, symname, pool);
+    if (rv == APR_SUCCESS || rv == APR_EINIT) { /* previously loaded?!? */
+        *driver = symbol;
+        name = apr_pstrdup(pool, name);
+        apr_hash_set(drivers, name, APR_HASH_KEY_STRING, *driver);
+        rv = APR_SUCCESS;
+        if ((*driver)->init) {
+            (*driver)->init(pool);
         }
-        goto unlock;
     }
-    *driver = symbol;
-    if ((*driver)->init) {
-        (*driver)->init(pool);
-    }
-    name = apr_pstrdup(pool, name);
-    apr_hash_set(drivers, name, APR_HASH_KEY_STRING, *driver);
-
-unlock:
     apu_dso_mutex_unlock();
 
 #else /* not builtin and !APR_HAS_DSO => not implemented */
@@ -369,7 +384,7 @@ APU_DECLARE(int) apr_dbd_prepare(const apr_dbd_driver_t *driver,
     /* find the number of parameters in the query */
     for (q = query; *q; q++) {
         if (q[0] == '%') {
-            if (isalpha(q[1])) {
+            if (apr_isalpha(q[1])) {
                 nargs++;
             } else if (q[1] == '%') {
                 q++;
@@ -385,7 +400,7 @@ APU_DECLARE(int) apr_dbd_prepare(const apr_dbd_driver_t *driver,
 
     for (p = pq, q = query, i = 0; *q; q++) {
         if (q[0] == '%') {
-            if (isalpha(q[1])) {
+            if (apr_isalpha(q[1])) {
                 switch (q[1]) {
                 case 'd': t[i] = APR_DBD_TYPE_INT;   break;
                 case 'u': t[i] = APR_DBD_TYPE_UINT;  break;
@@ -428,7 +443,7 @@ APU_DECLARE(int) apr_dbd_prepare(const apr_dbd_driver_t *driver,
                         case 'c': t[i] = APR_DBD_TYPE_CLOB;       q += 2; break;
                         case 'n': t[i] = APR_DBD_TYPE_NULL;       q += 2; break;
                         }
-                    } 
+                    }
                     break;
                 }
                 q++;
