@@ -29,12 +29,16 @@
 static apr_status_t file_cleanup(apr_file_t *file, int is_child)
 {
     apr_status_t rv = APR_SUCCESS;
+    int fd = file->filedes;
 
-    if (close(file->filedes) == 0) {
-        file->filedes = -1;
+    /* Set file descriptor to -1 before close(), so that there is no
+     * chance of returning an already closed FD from apr_os_file_get().
+     */
+    file->filedes = -1;
 
+    if (close(fd) == 0) {
         /* Only the parent process should delete the file! */
-        if (!is_child && (file->flags & APR_DELONCLOSE)) {
+        if (!is_child && (file->flags & APR_FOPEN_DELONCLOSE)) {
             unlink(file->fname);
         }
 #if APR_HAS_THREADS
@@ -44,6 +48,9 @@ static apr_status_t file_cleanup(apr_file_t *file, int is_child)
 #endif
     }
     else {
+        /* Restore, close() was not successful. */
+        file->filedes = fd;
+
         /* Are there any error conditions other than EINTR or EBADF? */
         rv = errno;
     }
@@ -93,37 +100,37 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
     apr_status_t rv;
 #endif
 
-    if ((flag & APR_READ) && (flag & APR_WRITE)) {
+    if ((flag & APR_FOPEN_READ) && (flag & APR_FOPEN_WRITE)) {
         oflags = O_RDWR;
     }
-    else if (flag & APR_READ) {
+    else if (flag & APR_FOPEN_READ) {
         oflags = O_RDONLY;
     }
-    else if (flag & APR_WRITE) {
+    else if (flag & APR_FOPEN_WRITE) {
         oflags = O_WRONLY;
     }
     else {
         return APR_EACCES; 
     }
 
-    if (flag & APR_CREATE) {
-        oflags |= O_CREAT; 
-        if (flag & APR_EXCL) {
+    if (flag & APR_FOPEN_CREATE) {
+        oflags |= O_CREAT;
+        if (flag & APR_FOPEN_EXCL) {
             oflags |= O_EXCL;
         }
     }
-    if ((flag & APR_EXCL) && !(flag & APR_CREATE)) {
+    if ((flag & APR_FOPEN_EXCL) && !(flag & APR_FOPEN_CREATE)) {
         return APR_EACCES;
     }   
 
-    if (flag & APR_APPEND) {
+    if (flag & APR_FOPEN_APPEND) {
         oflags |= O_APPEND;
     }
-    if (flag & APR_TRUNCATE) {
+    if (flag & APR_FOPEN_TRUNCATE) {
         oflags |= O_TRUNC;
     }
 #ifdef O_BINARY
-    if (flag & APR_BINARY) {
+    if (flag & APR_FOPEN_BINARY) {
         oflags |= O_BINARY;
     }
 #endif
@@ -131,7 +138,7 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
 #ifdef O_CLOEXEC
     /* Introduced in Linux 2.6.23. Silently ignored on earlier Linux kernels.
      */
-    if (!(flag & APR_FILE_NOCLEANUP)) {
+    if (!(flag & APR_FOPEN_NOCLEANUP)) {
         oflags |= O_CLOEXEC;
 }
 #endif
@@ -139,13 +146,13 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
 #if APR_HAS_LARGE_FILES && defined(_LARGEFILE64_SOURCE)
     oflags |= O_LARGEFILE;
 #elif defined(O_LARGEFILE)
-    if (flag & APR_LARGEFILE) {
+    if (flag & APR_FOPEN_LARGEFILE) {
         oflags |= O_LARGEFILE;
     }
 #endif
 
 #if APR_HAS_THREADS
-    if ((flag & APR_BUFFERED) && (flag & APR_XTHREAD)) {
+    if ((flag & APR_FOPEN_BUFFERED) && (flag & APR_FOPEN_XTHREAD)) {
         rv = apr_thread_mutex_create(&thlock,
                                      APR_THREAD_MUTEX_DEFAULT, pool);
         if (rv) {
@@ -163,15 +170,17 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
     if (fd < 0) {
        return errno;
     }
-    if (!(flag & APR_FILE_NOCLEANUP)) {
+    if (!(flag & APR_FOPEN_NOCLEANUP)) {
         int flags;
 
         if ((flags = fcntl(fd, F_GETFD)) == -1)
             return errno;
 
-        flags |= FD_CLOEXEC;
-        if (fcntl(fd, F_SETFD, flags) == -1)
-            return errno;
+        if ((flags & FD_CLOEXEC) == 0) {
+            flags |= FD_CLOEXEC;
+            if (fcntl(fd, F_SETFD, flags) == -1)
+                return errno;
+        }
     }
 
     (*new) = (apr_file_t *)apr_pcalloc(pool, sizeof(apr_file_t));
@@ -182,13 +191,13 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
     (*new)->fname = apr_pstrdup(pool, fname);
 
     (*new)->blocking = BLK_ON;
-    (*new)->buffered = (flag & APR_BUFFERED) > 0;
+    (*new)->buffered = (flag & APR_FOPEN_BUFFERED) > 0;
 
     if ((*new)->buffered) {
         (*new)->buffer = apr_palloc(pool, APR_FILE_DEFAULT_BUFSIZE);
         (*new)->bufsize = APR_FILE_DEFAULT_BUFSIZE;
 #if APR_HAS_THREADS
-        if ((*new)->flags & APR_XTHREAD) {
+        if ((*new)->flags & APR_FOPEN_XTHREAD) {
             (*new)->thlock = thlock;
         }
 #endif
@@ -211,7 +220,7 @@ APR_DECLARE(apr_status_t) apr_file_open(apr_file_t **new,
      */
     (*new)->pollset = NULL;
 #endif
-    if (!(flag & APR_FILE_NOCLEANUP)) {
+    if (!(flag & APR_FOPEN_NOCLEANUP)) {
         apr_pool_cleanup_register((*new)->pool, (void *)(*new), 
                                   apr_unix_file_cleanup, 
                                   apr_unix_child_file_cleanup);
@@ -264,8 +273,8 @@ APR_DECLARE(apr_status_t) apr_os_file_put(apr_file_t **file,
     (*file)->timeout = -1;
     (*file)->ungetchar = -1; /* no char avail */
     (*file)->filedes = *dafile;
-    (*file)->flags = flags | APR_FILE_NOCLEANUP;
-    (*file)->buffered = (flags & APR_BUFFERED) > 0;
+    (*file)->flags = flags | APR_FOPEN_NOCLEANUP;
+    (*file)->buffered = (flags & APR_FOPEN_BUFFERED) > 0;
 
 #ifndef WAITIO_USES_POLL
     /* Start out with no pollset.  apr_wait_for_io_or_timeout() will
@@ -278,7 +287,7 @@ APR_DECLARE(apr_status_t) apr_os_file_put(apr_file_t **file,
         (*file)->buffer = apr_palloc(pool, APR_FILE_DEFAULT_BUFSIZE);
         (*file)->bufsize = APR_FILE_DEFAULT_BUFSIZE;
 #if APR_HAS_THREADS
-        if ((*file)->flags & APR_XTHREAD) {
+        if ((*file)->flags & APR_FOPEN_XTHREAD) {
             apr_status_t rv;
             rv = apr_thread_mutex_create(&((*file)->thlock),
                                          APR_THREAD_MUTEX_DEFAULT, pool);
@@ -305,7 +314,7 @@ APR_DECLARE(apr_status_t) apr_file_open_flags_stderr(apr_file_t **thefile,
 {
     int fd = STDERR_FILENO;
 
-    return apr_os_file_put(thefile, &fd, flags | APR_WRITE, pool);
+    return apr_os_file_put(thefile, &fd, flags | APR_FOPEN_WRITE, pool);
 }
 
 APR_DECLARE(apr_status_t) apr_file_open_flags_stdout(apr_file_t **thefile, 
@@ -314,7 +323,7 @@ APR_DECLARE(apr_status_t) apr_file_open_flags_stdout(apr_file_t **thefile,
 {
     int fd = STDOUT_FILENO;
 
-    return apr_os_file_put(thefile, &fd, flags | APR_WRITE, pool);
+    return apr_os_file_put(thefile, &fd, flags | APR_FOPEN_WRITE, pool);
 }
 
 APR_DECLARE(apr_status_t) apr_file_open_flags_stdin(apr_file_t **thefile, 
@@ -323,7 +332,7 @@ APR_DECLARE(apr_status_t) apr_file_open_flags_stdin(apr_file_t **thefile,
 {
     int fd = STDIN_FILENO;
 
-    return apr_os_file_put(thefile, &fd, flags | APR_READ, pool);
+    return apr_os_file_put(thefile, &fd, flags | APR_FOPEN_READ, pool);
 }
 
 APR_DECLARE(apr_status_t) apr_file_open_stderr(apr_file_t **thefile, 
@@ -351,7 +360,7 @@ APR_IMPLEMENT_INHERIT_SET(file, flags, pool, apr_unix_file_cleanup)
  * suitable on Unix (see PR 41119). */
 APR_DECLARE(apr_status_t) apr_file_inherit_unset(apr_file_t *thefile)
 {
-    if (thefile->flags & APR_FILE_NOCLEANUP) {
+    if (thefile->flags & APR_FOPEN_NOCLEANUP) {
         return APR_EINVAL;
     }
     if (thefile->flags & APR_INHERIT) {
@@ -374,3 +383,13 @@ APR_DECLARE(apr_status_t) apr_file_inherit_unset(apr_file_t *thefile)
 }
 
 APR_POOL_IMPLEMENT_ACCESSOR(file)
+
+APR_DECLARE(apr_status_t) apr_file_link(const char *from_path, 
+                                          const char *to_path)
+{
+    if (link(from_path, to_path) == -1) {
+        return errno;
+    }
+
+    return APR_SUCCESS;
+}
